import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.image.BufferedImage;

/**
 * A class representing a Card that can be drawn on a canvas. Cards can be drawn face up or face down.
 * 
 * @author Joel Ross
 */
public class DrawableCard extends Card
{
	public static final int WIDTH = 72; //width of each card
	public static final int HEIGHT = 96; //height of each card

	private BufferedImage facePic;
	private int width;
	private int height;
	private boolean faceUp;

	/**
	 * Creates a new DrawableCard object with the given rank, suit, and picture for a face
	 * @param rank an integer representing the rank (1 to 13).
	 * @param suit an integer representing the suit (1 to 4). Use CLUBS, DIAMONDS, HEARTS, SPADES as options
	 * @param facePic A picture for the card's face
	 */
	public DrawableCard(int rank, int suit, BufferedImage facePic)
	{
		super(rank, suit);
		this.facePic = facePic;
		this.width = facePic.getWidth();
		this.height = facePic.getHeight();
		this.faceUp = true;
	}

	/**
	 * Draws the card onto the given graphics context at the specified position. The card is not drawn as selected
	 * @param g2d The graphics context on which to draw the card
	 * @param x The x-coordinate of the card's upper-left corner
	 * @param y The y-coordinate of the card's upper-left corner
	 */
	public void draw(Graphics2D g2d, int x, int y)
	{
		this.draw(g2d, x, y, false); //draw not selected
	}

	/**
	 * Draws the card onto the given graphics context at the specified position
	 * @param g2d The graphics context on which to draw the card
	 * @param x The x-coordinate of the card's upper-left corner
	 * @param y The y-coordinate of the card's upper-left corner
	 * @param selected Whether or not the card should be drawn with a "selected" highlight
	 */
	public void draw(Graphics2D g2d, int x, int y, boolean selected)
	{
		if(faceUp)
			drawFront(g2d, x, y);
		else
			drawBack(g2d, x, y);
		if(selected)
			drawSelectionBox(g2d, x, y);
	}

	/**
	 * A helper method for drawing the front of the card
	 * @param g2d The graphics context on which to draw the card
	 * @param x The x-coordinate of the card's upper-left corner
	 * @param y The y-coordinate of the card's upper-left corner
	 */
	private void drawFront(Graphics2D g2d, int x, int y)
	{
		g2d.drawImage(facePic, x, y, null); //very simple draw, since just putting the picture there!
	}


	/**
	 * A helper method for drawing the back of the card
	 * @param g2d The graphics context on which to draw the card
	 * @param x The x-coordinate of the card's upper-left corner
	 * @param y The y-coordinate of the card's upper-left corner
	 */
	private void drawBack(Graphics2D g2d, int x, int y)
	{
		Stroke oldStroke = g2d.getStroke(); //store old settings
		Paint oldPaint = g2d.getPaint();

		g2d.setColor(Color.BLUE); //make the back blue
		g2d.fillRect(x, y, width, height);
		
		g2d.setColor(Color.BLACK); //draw a black outline
		g2d.setStroke(new BasicStroke(1));
		g2d.drawRect(x, y, width, height);

		g2d.setStroke(oldStroke); //reset old settings
		g2d.setPaint(oldPaint);
	}

	/**
	 * A helper method for drawing the selection highlighting around the card
	 * @param g2d The graphics context on which to draw the card
	 * @param x The x-coordinate of the card's upper-left corner
	 * @param y The y-coordinate of the card's upper-left corner
	 */
	private void drawSelectionBox(Graphics2D g2d, int x, int y)
	{
		Stroke oldStroke = g2d.getStroke(); //store old settings
		g2d.setStroke(new BasicStroke(2));
		Paint oldPaint = g2d.getPaint();
		g2d.setPaint(Color.YELLOW);
		g2d.drawRect(x, y, WIDTH, HEIGHT);
		g2d.setStroke(oldStroke); //reset old settings
		g2d.setPaint(oldPaint);
	}
	
	/**
	 * Returns whether the card is face up or not
	 * @return Whether the card is face up
	 */
	public boolean isFaceUp()
	{
		return faceUp;
	}

	/**
	 * Flips the card over--if it was face up, it is now face down, and vice versa
	 */
	public void flip()
	{
		faceUp = !faceUp; //reverse if we're face up
	}

	public String toString()
	{
		String f;
		if(faceUp)
			f = "face up";
		else
			f = "face down";
		return super.toString() + " " + f;
	}
}
